unit DChessBoard;

interface

uses
  DMath,
  DCore,
  DGraph;

type
TChessBoard = class;
// TChessPiece//{{{
TChessPiece = class
private
  FChessBoard: TChessBoard;
public
  constructor Create(ChessBoard: TChessBoard);
  property ChessBoard: TChessBoard read FChessBoard;
end;
//}}}
// TChessBoard//{{{
TChessBoard = class
protected
  FSize: TVec2i; //  
  FPos: TVec2f;
  FCellSize: TVec2f;
  FPieces: array of array of TChessPiece;
  FBorder: TTexture;
  FFull: array[0..1] of TTexture;
  function GetPiece(Pos: TVec2i): TChessPiece; // nil -  
  procedure SetPiece(Pos: TVec2i; Piece: TChessPiece); 
  function GetBoardSize: TVec2f;
public
  constructor Create(Size: TVec2i);
  destructor Destroy; override;
  procedure Render;
  function CellInside(Pos: TVec2i): Boolean;
  function MovePiece(A, B: TVec2i): TChessPiece; //  ,    B
  function PutPiece(A: TVec2i; Piece: TChessPiece): TChessPiece; 
  function GetCell(P: TVec2f): TVec2i;
  function GetCellRect(Pos: TVec2i): TRect;
  function GetCellCenter(Pos: TVec2i): TVec2f;
  property Size: TVec2i read FSize;
  property Pos: TVec2f read FPos write FPos;
  property CellSize: TVec2f read FCellSize write FCellSize;
  property BoardSize: TVec2f read GetBoardSize;
  property Pieces[Index: TVec2i]: TChessPiece read GetPiece 
                                              write SetPiece; default;
end;
//}}}

implementation

// TChessPiece//{{{
constructor TChessPiece.Create(ChessBoard: TChessBoard);
begin
  FChessBoard := ChessBoard;
end;
//}}}
// TChessBoard//{{{
function TChessBoard.GetPiece(Pos: TVec2i): TChessPiece;//{{{
begin
  if (0 <= Pos.X) and (Pos.X < FSize.X) and
     (0 <= Pos.Y) and (Pos.Y < FSize.Y) then
    Result := FPieces[Pos.X][Pos.Y]
  else
    Result := nil;
end;
//}}}
procedure TChessBoard.SetPiece(Pos: TVec2i; Piece: TChessPiece); //{{{
begin
  if (0 <= Pos.X) and (Pos.X < FSize.X) and
     (0 <= Pos.Y) and (Pos.Y < FSize.Y) then
    FPieces[Pos.X][Pos.Y] := Piece;
end;
//}}}
function TChessBoard.GetBoardSize: TVec2f;//{{{
begin
  Result := Vec2f(FSize.X*FCellSize.X, FSize.Y*FCellSize.Y);
end;
//}}}
constructor TChessBoard.Create;//{{{
  var
    I, J: Integer;
begin
  FSize := Size;
  SetLength(FPieces, FSize.X, FSize.Y);
  for I := 0 to FSize.X - 1 do
    for J := 0 to FSize.Y - 1 do
      FPieces[I][J] := nil;
  FPos := Vec2f(0, 0);
  FCellSize := Vec2f(64, 64);
  //FBorder := Core.Graph.LoadTex('DATA\tex\cellBorder.tga');
  FFull[0] := Core.Graph.LoadTex('DATA\tex\cellFull.tga');
  FFull[1] := Core.Graph.LoadTex('DATA\tex\cellFull2.tga');
  //FFull := Core.Graph.LoadTex('DATA\fonts\m256.bit');
end;

//}}}
destructor TChessBoard.Destroy;//{{{
  var
    I, J: Integer;
begin
  for I := 0 to FSize.X - 1 do
    for J := 0 to FSize.Y - 1 do
      FPieces[I][J].Free;
end;

//}}}
procedure TChessBoard.Render;//{{{
  var
    Coord: TVec2f;
    I, J: Integer;
begin
  for I := 0 to FSize.X - 1 do
    for J := 0 to FSize.Y - 1 do begin
      Coord := FPos + Vec2f(I*FCellSize.X, J*FCellSize.Y);
      Core.Graph.SetColor(1, 1, 1, 1);
      //Core.Graph.DrawRect(Rect(Coord, Coord + FCellSize));
      Core.Graph.SetBlendMode(bmAlpha);
      //Core.Graph.SetColor(126/255, 235/255, (1+Ord((I+J) mod 2 = 0))/2, 1);
      Core.Graph.SetColor(1.0, 1.0, 1.0, 1.0);
      Core.Graph.SetTexture(FFull[(I + J) mod 2]);
      Core.Graph.DrawTexture(Rect(Coord, Coord + FCellSize));
      Core.Graph.SetTexture(-1);
    end;
end;
//}}}
function TChessBoard.CellInside(Pos: TVec2i): Boolean;//{{{
begin
  Result := (0 <= Pos.X) and (Pos.X < FSize.X) and
            (0 <= Pos.Y) and (Pos.Y < FSize.Y);
end;
//}}}
function TChessBoard.MovePiece(A, B: TVec2i): TChessPiece;//{{{
begin
  Result := Pieces[B];
  Pieces[B] := Pieces[A];
  Pieces[A] := nil;
end;
//}}}
function TChessBoard.PutPiece; //{{{
begin
  Result := Pieces[A];
  Pieces[A] := Piece;
end;
//}}}
function TChessBoard.GetCell(P: TVec2f): TVec2i;//{{{
  var
    Delta: TVec2f;
begin
  Delta := P - FPos;
  Result := Vec2i(Trunc(Delta.X/FCellSize.X), Trunc(Delta.Y/FCellSize.Y));
  if Delta.X < 0 then
    Result.X := Result.X - 1;
  if Delta.Y < 0 then
    Result.Y := Result.Y - 1;
end;
//}}}
function TChessBoard.GetCellRect(Pos: TVec2i): TRect;//{{{
begin
  Result.A := FPos + Vec2f(Pos.X*FCellSize.X, Pos.Y*FCellSize.Y);
  Result.B := Result.A + FCellSize; 
end;
//}}}
function TChessBoard.GetCellCenter(Pos: TVec2i): TVec2f;//{{{
begin
  Result := FPos + Vec2f((Pos.X+0.5)*FCellSize.X, (Pos.Y+0.5)*FCellSize.Y);
end;
//}}}
//}}}

end.
