{$MACRO ON}
unit DGame;

interface

uses
  DUtils,
  DDebug,
  DMath,
  DIntersection,
  DEvents,
  DWinApi,
  DSound,
  DGraph,
  DTexBuffer,
  DFont,
  DCamera2D,
  
  DCore,
  DChessBoard,
  DAI,
  DMenu,
  
  DGuiCore,
  DGuiCommon,
  DGuiBuilder;

type
TPlayer = class
end;
// TBalloon//{{{
TBalloon = class(TChessPiece)
private
  FPlayer: Integer; // 0, 1
  FPos: TVec2f;
  FDelta: TVec2f;
  FNextPos: TVec2f;
  FMoving: Boolean;
  FTex: TTexture;
public
  constructor Create(ChessBoard: TChessBoard; Player: Integer; Pos: TVec2f);
  procedure MoveTo(Pos: TVec2f);
  procedure Update;
  procedure Render;
  property Player: Integer read FPlayer;
  property Pos: TVec2f read FPos;
end;
//}}}
{$REGION 'TCell'}
TBugState = (bsBase, bsBug, bsFund);
TCell = record
  Bug: Boolean; //     
  Player: Integer; // ,   
  State: TBugState; //    - //
  Connected: Boolean; //     
  Depth: Integer; //    
  Last: TVec2i; //      
  CanPut: Boolean; //      
end;
{$ENDREGION}
//{{{ TScene
TScene = record
  Name: String;
  Start: TProcOfObj;
  Finish: TProcOfObj;
  Render: TProcOfObj;
  Update: TProcOfObj;
end;
//}}}
//{{{ TGame
TGame = class(TWinApp)
private
  FTime: Integer;
  FScenes: array of TScene;
  FCurScene: Integer;
  FNextScene: Integer;
  FLastFPS: Integer;
  FFontMan: TFontMan;
  FFont: TFont;
  FFixed: TFont;
  FBuffer: TTexBuffer;
  FChessBoard: TChessBoard;
  FBalloons: Array Of TBalloon;

  FAction: Boolean;
  FStart: TVec2i;
  FFinish: TVec2i;
  FDelta: TVec2i;

  FCurPlayer: Integer;

  FGui: TGui;
  FGuiBuilder: TGuiBuilder;
  FLabels: array of TGuiLabel;
  FBugsLabel: TGuiMenuButton;
  FMenu: TGuiMenu;
  FBar: TGuiMenuValueBar;
  FRadio: TGuiMenuRadioBox;
  FAuthor: TGuiMenuWindow;
  FHelp: TGuiMenuWindow;
  FMap: array[0..7] of array[0..7] of TCell;
  FBugsCount: Integer;
  FAI: array[0..1] of TAI;
  FGameType: Integer;
  FWinner: Integer;
  
  // 
  FTextures: array[0..1] of array[TBugState] of array[Boolean] of TTexture;

  function GetTime: Integer;
  procedure AddScene(Name: String; Start, Finish, Render, Update: TProcOfObj);
  procedure GameStart;
  procedure GameFinish;
  procedure GameUpdate;
  procedure GameRender;
  procedure CreateGui;
  function GetCell(Pos: TVec2i): TCell; inline;
  procedure OnSetVolume;
  procedure OnNewGame;
  procedure CheckWinner;
  procedure OnWin;
  procedure OnRound;
  procedure UpdateGui;
  procedure OnBugsClick;
  procedure OnAuthor;
  procedure OnHelp;
public
  procedure GoToScene(Name: String);
  constructor Create;
  destructor Destroy; override;
  procedure Render; override;
  procedure Update; override;
  procedure OnEvent(const Event: TEvent); override;
  procedure KeyPressed(Key: Byte);
  procedure MouseDown(Key: TMouseButton; Mouse: TVec2f);
  procedure MouseUp(Key: TMouseButton; Mouse: TVec2f);
  procedure DrawText(X, Y: Single; AX, AY: Single; Text: String);
  procedure AddBalloon(Player: Integer; Pos: TVec2i);
  procedure DoStep;
  procedure StartRound;
  function InMap(Pos: TVec2i): Boolean;
  function GetConnected(Player: Integer; Pos: TVec2i; var Last: TVec2i): Boolean;
  procedure RecalcMap;
  procedure RecalcCanPut;
  function CanPut(const Pos: TVec2i): Boolean;
  procedure Put(Pos: TVec2i);
  function GetPlayerColor(Player: Integer): TVec4f;
  property Cells[Index: TVec2i]: TCell read GetCell;
  property BugsCount: Integer read FBugsCount;
  property Time: Integer read GetTime;
  property CurPlayer: Integer read FCurPlayer;
end;
//}}}

var
  Game: TGame;

implementation

// TBalloon {{{
constructor TBalloon.Create(ChessBoard: TChessBoard; Player: Integer; Pos: TVec2f);//{{{
begin
  inherited Create(ChessBoard);
  FPlayer := Player;
  FPos := Pos;
  if FPlayer = 0 then
    FTex := Core.Graph.LoadTex('DATA\tex\sphere_yellow.tga')
  else
    FTex := Core.Graph.LoadTex('DATA\tex\sphere_green.tga');
end;
//}}}
procedure TBalloon.MoveTo(Pos: TVec2f);//{{{
begin
  FNextPos := Pos;
  FDelta := Norm2f(FNextPos - FPos);
  FMoving := True;
end;
//}}}
procedure TBalloon.Update;//{{{
begin
  if not FMoving then
    Exit;
  FPos := FPos + FDelta*5;
  if Dot2f(FNextPos - FPos, FDelta) <= 0 then begin
    FMoving := False;
    FPos := FNextPos;
  end;
end;
//}}}
procedure TBalloon.Render;//{{{
begin
  if FPlayer = 0 then
    Core.Graph.SetColor(3/4, 3/4,   0, 0.5)
  else
    Core.Graph.SetColor(3/4,   0, 3/4, 0.5);
  //Core.Graph.DrawCircle(FPos, 16, True);
  Core.Graph.SetBlendMode(bmAlpha);
  Core.Graph.SetColor(1, 1, 1, 1);
  Core.Graph.SetTexture(FTex);
  Core.Graph.DrawTexture(Rect(FPos - Vec2f(32, 32), FPos + Vec2f(32, 32)));
  Core.Graph.SetTexture(-1);
end;
//}}}
//}}}
//{{{ TGame
function TGame.GetTime: Integer;//{{{
begin
  Result := GetTimer - FTime;
end;
//}}}
procedure TGame.AddScene;//{{{
begin
  SetLength(FScenes, Length(FScenes) + 1);
  FScenes[High(FScenes)].Name := Name;
  FScenes[High(FScenes)].Start := Start;
  FScenes[High(FScenes)].Finish := Finish;
  FScenes[High(FScenes)].Render := Render;
  FScenes[High(FScenes)].Update := Update;
end;
//}}}
function TGame.GetCell(Pos: TVec2i): TCell; inline;
begin
  Result := FMap[Pos.X][Pos.Y];
end;

procedure TGame.OnSetVolume;
begin
  // 8)        DMusic.pas
  Core.Sound.MusicVolume := (sqrt(sqrt(sqrt(FBar.Value))));
end;

procedure TGame.OnNewGame;
begin
  StartRound;
end;

procedure TGame.CheckWinner;
  var
    I, J: Integer;
begin
  if FWinner <> -1 then
    Exit;
  for I := 0 to 7 do
    for J := 0 to 7 do
      if CanPut(Vec2i(I, J)) then
        Exit;
  FWinner := 1 - FCurPlayer;
  OnWin;
end;

procedure TGame.OnWin;
begin
  if FGameType = 0 then  begin
    if FWinner = 1 then begin
      Core.Sound.PlaySoundMMSystem('DATA\sound\lose.wav');
      FLabels[0]._Label.Text := '.';
      FLabels[1]._Label.Text := '? ;)';
    end else begin
      Core.Sound.PlaySoundMMSystem('DATA\sound\win.wav');
      FLabels[0]._Label.Text := '!';
      FLabels[1]._Label.Text := ' ?';
    end;
    FLabels[0].Color := FLabels[0].Style.PrimaryColor;
    FLabels[1].Color := FLabels[0].Style.PrimaryColor;
    FBugsLabel._Label.Color := FLabels[0].Style.PrimaryColor;
    FBugsLabel._Label.Text := '!';
  end else begin
    Core.Sound.PlaySoundMMSystem('DATA\sound\win.wav');
    FLabels[0]._Label.Text := ' ' + IntToStr(FWinner + 1);
    FLabels[1]._Label.Text := '!';
    FLabels[0].Color := GetPlayerColor(FWinner);
    FLabels[1].Color := GetPlayerColor(FWinner);
    FBugsLabel._Label.Text := '';
    FBugsLabel._Label.Color := GetPlayerColor(FWinner);
  end;
end;

procedure TGame.OnRound;
begin
  FLabels[0]._Label.Text := '';
  FLabels[1]._Label.Text := '';
  if FGameType = 0 then begin
    FAI[0].ThinkTime := 100;
    FAI[1].ThinkTime := 100;
  end else if FGameType = 2 then begin
    FAI[0].ThinkTime := 300;
    FAI[1].ThinkTime := 300;
  end;
end;

procedure TGame.UpdateGui;
begin
  FBugsLabel._Label.Text := IntToStr(FBugsCount);
  FBugsLabel._Label.Color := GetPlayerColor(FCurPlayer);
  FLabels[0].Color := GetPlayerColor(FCurPlayer);
  FLabels[1].Color := GetPlayerColor(FCurPlayer);
end;

procedure TGame.OnBugsClick;
begin
  if FWinner = -1 then
    Exit;
  OnNewGame;
end;

procedure TGame.OnAuthor;
begin
  FAuthor.Show := True;
end;

procedure TGame.OnHelp;
begin
  FHelp.Show := True;
end;

procedure TGame.GoToScene(Name: String);//{{{
  var
    I: Integer;
begin
  for I := 0 to High(FScenes) do
    if FScenes[I].Name = Name then begin
      FNextScene := I;
      Exit;
    end;
  SDebug.Log('! Scene not found: ' + Name);
end;

//}}}
procedure TGame.GameStart;//{{{
begin
end;
//}}}
procedure TGame.GameFinish;//{{{
begin
end;
//}}}
procedure TGame.GameUpdate;//{{{
  var
    I: Integer;
begin
  if FGameType in [0, 2] then
    FAI[0].Update;
  if FGameType = 2 then
    FAI[1].Update;
  FGui.Update;
end;

//}}}
procedure TGame.GameRender;//{{{
  var
    I, J: Integer;
    Cell, D: TVec2i;
    R: TRect;
  function Test(A, B: TVec2i): Boolean;
  begin
    if (not InMap(A)) or (not InMap(B)) then
      Result := False
    else
      Result := Cells[A].Bug and Cells[B].Bug and 
                (Cells[A].Player = Cells[B].Player) and
                (Cells[B].Depth < Cells[A].Depth);
  end;
  function FindNearest(P: TVec2i; var R: TVec2i): Boolean;
    var
      D, I, J: Integer;
  begin
    Result := False;
    for I := -1 to 1 do
      for J := -1 to 1 do
        if I*I + J*J = 1 then
          if Test(P, P + Vec2i(I, J)) then
              if Result then begin
                if Cells[P + Vec2i(I, J)].Depth < Cells[R].Depth then
                  R := P + Vec2i(I, J);
              end else begin
                Result := True;
                R := P + Vec2i(I, J);
              end;
  end;
begin
  with Core.Graph, FChessBoard do begin
    Clear(0, 0, 0.3, 0);
    BeginRender2D(GWidth, GHeight);
    SetBlendMode(bmNo);
    FChessBoard.Render;
    SetLineWidth(5);
    //SetColor(0.5, 0.5, 0.5, 1.0);
    Color4f(GetPlayerColor(FCurPlayer)*(1/2) + Vec4f(0, 0, 0, 0.5));
    Cell := GetCell(Core.Window.Mouse);
    if CellInside(Cell) and (FWinner = -1) then
      if CanPut(Cell) then
        DrawRect(GetCellRect(Cell), False);
    if Cell.X + Cell.Y = 666 then
      SDebug.Log('I love free pascal >:=E');
    //for I := 0 to High(FBalloons) do
    //  FBalloons[I].Render;
    SetLineWidth(10);
    for I := 0 to 7 do
      for J := 0 to 7 do
        with FMap[I][J] do
          if Bug and Connected then begin
            Core.Graph.Color4f(GetPlayerColor(Player));
            if State <> bsBase then begin
              D := Last - Vec2i(I, J);
              if Dot2i(D, D) <> 1 then 
                if FindNearest(Vec2i(I, J), D) then begin
                  DrawLine(Vec2i(I, J), D);
                  Continue
                end;
              DrawLine(GetCellCenter(Vec2i(I, J)),
                       GetCellCenter(Last));
            end;
          end;
    SetLineWidth(1);
    for I := 0 to 7 do
      for J := 0 to 7 do
        with FMap[I][J] do
          if Bug then begin
            R := GetCellRect(Vec2i(I, J));
            {Core.Graph.Color4f(GetPlayerColor(FMap[I][J].Player));
            case State of
              bsBase: DrawRect(Rect(R.A+Vec2f(10, 10), R.B-Vec2f(10, 10)), True);
              bsBug: DrawRect(Rect(2/3*R.A+1/3*R.B, 1/3*R.A+2/3*R.B));
              bsFund: DrawRect(Rect(3/4*R.A+1/4*R.B, 1/4*R.A+3/4*R.B));
            end;}
            Color4f(Vec4f(1, 1, 1, 1));
            SetBlendMode(bmAlpha);
            SetTexture(FTextures[Player][State][Connected]);
            DrawTexture(R);
          end;
    (*FBuffer.FinishRender;
      FBuffer.StartRender;
        Clear(Random, Random, Random, 1);
      EndRender;
    //Core.Window.SwapBuffers;
    FBuffer.FinishRender;

    BeginRender2D(GWidth, GHeight);
    SetBlendMode(bmNo);
    if FBuffer.Ready then begin
      SetColor(1.0, 1.0, 0.0, 1.0);
      DrawRect(Rect(Vec2f(0, 0), Vec2f(GWidth, GHeight)));
      FBuffer.Bind;
      SetColor(1.0, 1.0, 1.0, 1.0);
      DrawTexture(Rect(Vec2f(GWidth*0.25, GHeight*0.25), 
                       Vec2f(GWidth*0.75, GHeight*0.75)));
      FBuffer.Unbind;
    end;
    if Core.Key(Ord('3')) then begin
      SetBlendMode(bmAlpha);
      SetColor(0.9, 0.9, 0.9, 1.0);
    end;
    //EndRender;
    FBuffer.StartRender;*)
    FGui.Render;
  end;
end;

//}}}
procedure TGame.CreateGui;
  var
    Style: TGuiStyle;
    Button: TGuiMenuButton;
    Window: TGuiMenuWindow;
begin
  FGui := TGui.Create(Core.Graph, Core.Window);
  Style := TGuiStyle.Create(FGui, 'woodworker');
  with Style do begin
    PrimaryColor := Vec4f(0.75, 0.75, 0.75, 1.0);
    //BackColor := Vec4f(71/255, 67/255, 26/255, 1.0);
    BackColor := Vec4f(22/255, 117/255, 23/255, 1.0);
    AddFont('default', 'DATA\fonts\c256.bit', -6);
    AddFont('large', 'DATA\fonts\c1024.bit', -24);
  end;
  FGui.AddStyle(Style);
  FGuiBuilder := TGuiBuilder.Create(FGui);
  FGuiBuilder.SetStyle('woodworker');
  SetLength(FLabels, 2);
  FLabels[0] := FGuiBuilder._Label(Vec2f(20, 250), Vec2f(100, 10), '');
  FLabels[1] := FGuiBuilder._Label(Vec2f(20, 265), Vec2f(100, 10), '');
  FBugsLabel := TGuiMenuButton.Create(FGui, 'woodworker', '3');
  FBugsLabel.Local := Vec2f(10, 280);
  FBugsLabel.Size := Vec2f(100, 60);
  FBugsLabel.Parent := FGui.Root;
  FBugsLabel._Label.Font := 'large';
  FBugsLabel.OnClick := OnBugsClick;
  FMenu := TGuiMenu.Create(FGui, 'woodworker_menu', 'woodworker');
  FGui.Link(FGui.Root, FMenu);
  FMenu.Size := Vec2f(140, 250);
  FMenu.Local := Vec2f(GWidth - 40, 0);
  FRadio := TGuiMenuRadioBox.Create(FGui, 'woodworker');
  FRadio.Parent := FMenu;
  FRadio.Local := Vec2f(20, 10);
  FRadio.Size := Vec2f(100, 45);
  FRadio.Items.Add(' ');
  FRadio.Items.Add('2 ');
  FRadio.Items.Add('2 ');
  Button := TGuiMenuButton.Create(FGui, 'woodworker', '');
  FGui.Link(FMenu, Button);
  Button.Local := Vec2f(0, 60);
  Button.Size := Vec2f(FMenu.Size.X, 20);
  Button._Label.Align := Vec2f(0.5, 0.5);
  //Button._Label.Color := Vec4f(0.9, 0.9, 0.9, 1.0);
  Button._Label.Font := 'default';
  Button.OnClick := OnNewGame;
  Button := TGuiMenuButton.Create(FGui, 'woodworker', '');
  FGui.Link(FMenu, Button);
  Button.Local := Vec2f(0, 100);
  Button.Size := Vec2f(FMenu.Size.X, 20);
  Button._Label.Align := Vec2f(0.5, 0.5);
  //Button._Label.Color := Vec4f(0.9, 0.9, 0.9, 1.0);
  Button._Label.Font := 'default';
  Button.OnClick := OnHelp;
  Button := TGuiMenuButton.Create(FGui, 'woodworker', '');
  FGui.Link(FMenu, Button);
  Button.Local := Vec2f(0, 120);
  Button.Size := Vec2f(FMenu.Size.X, 20);
  Button._Label.Align := Vec2f(0.5, 0.5);
  //Button._Label.Color := Vec4f(0.9, 0.9, 0.9, 1.0);
  Button._Label.Font := 'default';
  Button.OnClick := OnAuthor;
  FGuiBuilder.SetOwner(FMenu);
  FGuiBuilder._Label(Vec2f(0, 210), Vec2f(FMenu.Size.X, 20), ':');
  FBar := TGuiMenuValueBar.Create(FGui, 'woodworker');
  FBar.Parent := FMenu;
  FBar.Local := Vec2f(20, 230);
  FBar.Size := Vec2f(100, 15);
  FBar.OnChange := OnSetVolume;
  FAuthor := TGuiMenuWindow.Create(FGui, 'author_window', 'woodworker');
  FAuthor.Local := Vec2f(250, 200);
  FAuthor.Size := Vec2f(300, 200);
  FAuthor.Parent := FGui.Root;
  FGuiBuilder.SetOwner(FAuthor);
  FGuiBuilder._Label(Vec2f(0, 20), Vec2f(FAuthor.Size.X, 20), ' ');
  FGuiBuilder._Label(Vec2f(0, 40), Vec2f(FAuthor.Size.X, 20), ' ');
  FGuiBuilder._Label(Vec2f(0, 60), Vec2f(FAuthor.Size.X, 20), 'igdc.ru').Color := Vec4f(1, 1, 1, 1);
  FGuiBuilder._Label(Vec2f(0, 100), Vec2f(FAuthor.Size.X, 20), '');
  FGuiBuilder._Label(Vec2f(0, 120), Vec2f(FAuthor.Size.X, 20), '').Color := Vec4f(1, 1, 1, 1);
  FGuiBuilder._Label(Vec2f(0, 140), Vec2f(FAuthor.Size.X, 20), 'OnOff').Color := Vec4f(1, 1, 1, 1);
  FGuiBuilder._Label(Vec2f(0, 180), Vec2f(FAuthor.Size.X, 20), ' 2009');
  FAuthor.Show := False;
  FHelp := TGuiMenuWindow.Create(FGui, 'help_window', 'woodworker');
  FHelp.Local := Vec2f(150, 100);
  FHelp.Size := Vec2f(500, 400);
  FHelp.Parent := FGui.Root;
  FGuiBuilder.SetOwner(FHelp);
  FGuiBuilder._Label(Vec2f(0, 40), Vec2f(FHelp.Size.X, 20), '    ');
  FGuiBuilder._Label(Vec2f(0, 60), Vec2f(FHelp.Size.X, 20), '   .');
  FGuiBuilder._Label(Vec2f(0, 100), Vec2f(FHelp.Size.X, 20), ' ,     .');
  FGuiBuilder._Label(Vec2f(0, 120), Vec2f(FHelp.Size.X, 20), '      .');
  FGuiBuilder._Label(Vec2f(0, 140), Vec2f(FHelp.Size.X, 20), '    ');
  FGuiBuilder._Label(Vec2f(0, 160), Vec2f(FHelp.Size.X, 20), '    .');
  FGuiBuilder._Label(Vec2f(0, 180), Vec2f(FHelp.Size.X, 20), '     ');
  FGuiBuilder._Label(Vec2f(0, 200), Vec2f(FHelp.Size.X, 20), ',     .');
  FGuiBuilder._Label(Vec2f(0, 220), Vec2f(FHelp.Size.X, 20), '    ');
  FGuiBuilder._Label(Vec2f(0, 240), Vec2f(FHelp.Size.X, 20), ' .');
  //FGuiBuilder._Label(Vec2f(0, 300), Vec2f(FHelp.Size.X, 20), '!').Font := 'large';
  Button := TGuiMenuButton.Create(FGui, 'woodworker', '!');
  Button.Local := Vec2f(0, 290);
  Button.Size := Vec2f(FHelp.Size.X, 60);
  Button.Parent := FHelp;
  Button._Label.Font := 'large';
  Button.OnClick := FHelp.OnClose;
  if FileExist('already_run') then
    FHelp.Show := False
  else
    CreateFile('already_run');
//  FGuiBuilder.Button(Rect(Vec2f(40, 40), Vec2f(200, 200)), 'Test', nil);
end;

constructor TGame.Create;//{{{
  var
    I: Integer;
    B: Boolean;
    S: TBugState;
begin
  AddScene('game', GameStart, GameFinish, GameRender, GameUpdate);
  GoToScene('game');
  FCurScene := 0;
  FNextScene := -1;

  Core.Sound.SoundVolume := 1;
  Core.Sound.MusicVolume := 0.93;

  FFontMan := TFontMan.Create(Core.Graph);
  //FFont := FFontMan.CreateFont('DATA\fonts\FrankC.bit', -3);
  //FFixed := FFontMan.CreateFont('DATA\fonts\FixedSys.bit', -3);

  FChessBoard := TChessBoard.Create(Vec2i(8, 8));
  FChessBoard.Pos := (Core.Window.Size - FChessBoard.BoardSize)/2;
  {
  for I := 0 to 5 do begin
    AddBalloon(0, Vec2i(1 + I, 1));
    AddBalloon(1, Vec2i(1 + I, 6));
  end;
  }
  CreateGui;

  FBuffer := TTexBuffer.Create(Core.Graph, 512, 512, TB_Alpha or TB_Texture2D or TB_MipMap);
  
  for I := 0 to 1 do
    for S := bsBase to bsFund do
      for B := False to True do
        FTextures[I][S][B] := Core.Graph.LoadTex('DATA\diods\diod_' + 
                                                    IntToStr(I) +
                                                 '_' + IntToStr(Ord(S)) + 
                                                 '_' + IntToStr(Ord(B)) + 
                                                 '.tga');

  FAI[0] := TAI.Create(1);
  FAI[1] := TAI.Create(0);
  FGameType := 0;

  StartRound;
  RecalcMap;

  Core.Sound.PlayMusic('DATA\music\music00.xm');
end;
//}}}
destructor TGame.Destroy;//{{{
begin
  Core.Sound.PlayMusic('');
  FAI[0].Free;
  FAI[1].Free;
  FChessBoard.Free;
  //FFont.Free;
  FFontMan.Free;
end;
//}}}
procedure TGame.Render;//{{{
begin
  with Core.Graph do begin
    //Clear(214/255, 168/255, 135/255);
    //BeginRender2D(GWidth, GHeight);
    FScenes[FCurScene].Render;
    //EndRender;
  end;
end;
//}}}
procedure TGame.Update;//{{{
  var
    I: Integer;
begin
  Core.Update;

  for I := 0 to High(FBalloons) do
    FBalloons[I].Update;

  if FNextScene <> -1 then begin
    FScenes[FCurScene].Finish;
    FScenes[FNextScene].Start;
    FCurScene := FNextScene;
    FNextScene := -1;
  end;
  
  FScenes[FCurScene].Update;

  if FLastFPS <> Core.WinApi.FPS then
    Core.Window.Caption := GName + ' (FPS: ' + IntToStr(Core.WinApi.FPS) + ')';
  FLastFPS := Core.WinApi.FPS;
end;
//}}}
procedure TGame.OnEvent(const Event: TEvent);//{{{
begin
  if Event is TWinApiKey then
    if not Core.Key(TWinApiKey(Event).Key) then 
      KeyPressed(TWinApiKey(Event).Key);
  if (Event is TWinApiMouse) then
    if TWinApiMouse(Event).Down then
      MouseDown(TWinApiMouse(Event).Button, TWinApiMouse(Event).Pos)
    else
      MouseUp(TWinApiMouse(Event).Button, TWinApiMouse(Event).Pos);
  Core.OnEvent(Event);
  FGui.OnEvent(Event);
end;

//}}}
procedure TGame.KeyPressed(Key: Byte);//{{{
begin
end;

//}}}
procedure TGame.MouseDown(Key: TMouseButton; Mouse: TVec2f);//{{{
begin
  if Key = mbLeft then begin
    //FStart := FChessBoard.GetCell(Mouse);
    //FAction := FChessBoard.CellInside(FStart);
    if (FGameType = 1) or ((FGameType = 0) and (FCurPlayer = 0)) then
      Put(FChessBoard.GetCell(Mouse));
  end;
end;
//}}}
procedure TGame.MouseUp(Key: TMouseButton; Mouse: TVec2f);//{{{
begin
  if FAction and (Key = mbLeft) then begin
    FAction := False;
    FFinish := FChessBoard.GetCell(Mouse);
    if not FChessBoard.CellInside(FFinish) then
      Exit;
    FDelta := FFinish - FStart;
    if Dot2i(FDelta, FDelta) = 1 then
      DoStep;
  end;
end;
//}}}
procedure TGame.DrawText(X, Y: Single; AX, AY: Single; Text: String);//{{{
  var
    Pos: TVec2f;
begin
  Pos := Vec2f(X - AX*FFont.GetWidth(Text), Y - AY*FFont.GetHeight(Text));
  FFont.Draw(Pos, Text);
end;
//}}}
procedure TGame.AddBalloon(Player: Integer; Pos: TVec2i);//{{{
  var
    Balloon: TBalloon;
begin
  Balloon := TBalloon.Create(FChessBoard, Player, FChessBoard.GetCellCenter(Pos));
  FChessBoard[Pos] := Balloon;
  SetLength(FBalloons, Length(FBalloons) + 1);
  FBalloons[High(FBalloons)] := Balloon;
end;
//}}}
procedure TGame.DoStep;//{{{
  var
    Balloon: TBalloon;
begin
  Balloon := TBalloon(FChessBoard[FStart]);
  if Balloon = nil then
    Exit;
  //Balloon.MoveTo(FChessBoard.GetCellCenter(FStart + Delta));
  Balloon.MoveTo(FChessBoard.GetCellCenter(FStart + FDelta));
  Balloon := TBalloon(FChessBoard.MovePiece(FStart, FStart + FDelta));
  FStart := FStart + FDelta;

  while Balloon <> nil do begin
    Balloon.MoveTo(FChessBoard.GetCellCenter(FStart + FDelta));
    Balloon := TBalloon(FChessBoard.PutPiece(FStart + FDelta, Balloon));
    FStart := FStart + FDelta;
  end;
end;
{$IFNDEF FLAG_FPC}{$REGION 'TGame.InMap'}{$ENDIF}
function TGame.InMap(Pos: TVec2i): Boolean;
begin
  Result := (0 <= Pos.X)and(Pos.X <= 7)and(0 <= Pos.Y)and(Pos.Y <= 7);
end;
{$IFNDEF FLAG_FPC}{$ENDREGION}{$ENDIF}
{$IFNDEF FLAG_FPC}{$REGION 'TGame.GetConnected'}{$ENDIF}
function TGame.GetConnected(Player: Integer; Pos: TVec2i; var Last: TVec2i): Boolean;
begin
  if not InMap(Pos) then
    Result := False
  else
    Result := FMap[Pos.X][Pos.Y].Connected and
              (FMap[Pos.X][Pos.Y].Player = Player);
  if Result then
    Last := Pos;
end;
{$IFNDEF FLAG_FPC}{$ENDREGION}{$ENDIF}
{$IFNDEF FLAG_FPC}{$REGION 'TGame.RecalcMap'}{$ENDIF}
procedure TGame.RecalcMap;
  var
    I, J, K: Integer;
begin
  for I := 0 to 7 do
    for J := 0 to 7 do
      FMap[I][J].Connected := FMap[I][J].Bug and (FMap[I][J].State = bsBase);
  for K := 0 to 100 do
    for I := 0 to 7 do
      for J := 0 to 7 do
        if not FMap[I][J].Connected then 
          if FMap[I][J].Bug then
            with FMap[I][J] do begin
              Connected := GetConnected(Player, Vec2i(I  , J-1), Last) or
                           GetConnected(Player, Vec2i(I-1, J  ), Last) or
                           GetConnected(Player, Vec2i(I+1, J  ), Last) or
                           GetConnected(Player, Vec2i(I  , J+1), Last) or
                           GetConnected(Player, Vec2i(I-1, J-1), Last) or
                           GetConnected(Player, Vec2i(I+1, J-1), Last) or
                           GetConnected(Player, Vec2i(I-1, J+1), Last) or
                           GetConnected(Player, Vec2i(I+1, J+1), Last);
              if Connected then
                Depth := K;
            end;
end;
{$IFNDEF FLAG_FPC}{$ENDREGION}{$ENDIF}
{$IFNDEF FLAG_FPC}{$REGION 'TGame.RecalcCanPut'}{$ENDIF}
procedure TGame.RecalcCanPut;
var
  I, J: Integer;
  Temp: TVec2i;
begin
  for I := 0 to 7 do
    for J := 0 to 7 do
      FMap[I][J].CanPut := False;
  for I := 0 to 7 do
    for J := 0 to 7 do
      if (FMap[I][J].bug and (FMap[I][J].Player <> FCurPlayer) and 
          (FMap[I][J].State = bsBug)) or not FMap[I][J].Bug then
        if GetConnected(FCurPlayer, Vec2i(I-1, J-1), Temp) or
           GetConnected(FCurPlayer, Vec2i(I  , J-1), Temp) or
           GetConnected(FCurPlayer, Vec2i(I+1, J-1), Temp) or
           GetConnected(FCurPlayer, Vec2i(I-1, J  ), Temp) or
           GetConnected(FCurPlayer, Vec2i(I+1, J  ), Temp) or
           GetConnected(FCurPlayer, Vec2i(I-1, J+1), Temp) or
           GetConnected(FCurPlayer, Vec2i(I  , J+1), Temp) or
           GetConnected(FCurPlayer, Vec2i(I+1, J+1), Temp) then
          FMap[I][J].CanPut := True;
end;
{$IFNDEF FLAG_FPC}{$ENDREGION}{$ENDIF}
{$IFNDEF FLAG_FPC}{$REGION 'TGame.StartRound'}{$ENDIF}
procedure TGame.StartRound;
  var
    I, J: Integer;
begin
  Core.Sound.PlaySoundMMSystem('DATA\sound\newgame.wav');
  for I := 0 to 7 do
    for J := 0 to 7 do
      FMap[I][J].Bug := False;
  FMap[0][0].Bug := True;
  FMap[0][0].Player := 0;
  FMap[0][0].State := bsBase;
  FMap[7][7].Bug := True;
  FMap[7][7].Player := 1;
  FMap[7][7].State := bsBase;
  FCurPlayer := 0;
  FBugsCount := 3;
  RecalcMap;
  RecalcCanPut;
  FGameType := FRadio.Current;
  FWinner := -1;
  UpdateGui;
  OnRound;
end;
{$IFNDEF FLAG_FPC}{$ENDREGION}{$ENDIF}
{$IFNDEF FLAG_FPC}{$REGION 'TGame.CanPut}{$ENDIF}
function TGame.CanPut(const Pos: TVec2i): Boolean;
begin
  Result := False;
  if not InMap(Pos) then
    Exit;
  {if FMap[Pos.X][Pos.Y].Bug then
    if FMap[Pos.X][Pos.Y].Player = FCurPlayer then
      Result := False
    else if FMap[Pos.X][Pos.Y].State <> bsBug then
      Result := False
    else
      Result := True
  else
    Result := True;}
  //Result := True;
  Result := FMap[Pos.X][Pos.Y].CanPut;
end;
{$IFNDEF FLAG_FPC}{$ENDREGION}{$ENDIF}
{$IFNDEF FLAG_FPC}{$REGION 'TGame.Put'}{$ENDIF}
procedure TGame.Put(Pos: TVec2i);
begin
  if FWinner <> -1 then
    Exit;
  if not InMap(Pos) then
    Exit;
  if not CanPut(Pos) then begin
    Core.Sound.PlaySoundMMSystem('DATA\sound\imp.wav');
    Exit;
  end;
  FMap[Pos.X][Pos.Y].Player := FCurPlayer;
  if FMap[Pos.X][Pos.Y].Bug then begin
    Core.Sound.PlaySoundMMSystem('DATA\sound\eat.wav');
    FMap[Pos.X][Pos.Y].State := bsFund;
  end else begin
    FMap[Pos.X][Pos.Y].Bug := True;
    FMap[Pos.X][Pos.Y].State := bsBug
  end;
  Dec(FBugsCount);
  if FBugsCount = 0 then begin
    FBugsCount := 3;
    FCurPlayer := (FCurPlayer + 1) mod 2;
  end;
  RecalcMap;
  RecalcCanPut;
  UpdateGui;
  CheckWinner;
end;
{$IFNDEF FLAG_FPC}{$ENDREGION}{$ENDIF}
function TGame.GetPlayerColor;
begin
  //Result := Vec4f(1, 1*Ord(Player <> 0), 0.5*Ord(Player <> 0), 1.0)
  if Player = 0 then
    Result := Vec4f(0.8, 0.2, 0.2, 1.0)
  else
    Result := Vec4f(0.2, 0.8, 0.2, 1.0);
end;
//}}}
//}}}

initialization
end.
