unit DMenu;

interface

uses
  DDebug,
  DMath,
  DGuiCore,
  DGuiCommon,
  DWinApi,
  DUtils;

type
TGuiMenu = class(TWidget)
  constructor Create(Gui: TGui; Name, Style: String);
  procedure Render; override;
  procedure Update; override;
end;

TGuiMenuButton = class(TWidget)
private
  FLabel: TLabel;
  FOnClick: TProcOfObj;
public
  constructor Create(Gui: TGui; Style: String; Caption: String);
  procedure Render; override;
  function OnMouseDown(Button: TMouseButton; Pos: TVec2f; Flags: Cardinal): Boolean; override;
  property OnClick: TProcOfObj read FOnClick write FOnClick;
  property _Label: TLabel read FLabel;
end;

TGuiMenuRadioBox = class(TWidget)
private
  FItems: TListOfString;
  FCurrent: Integer;
  FFont: TGuiFont;
  FColor: TVec4f;
  function GetFontName: String;
  procedure SetFont(Name: String);
public
  constructor Create(Gui: TGui; Style: String);
  destructor Destroy; override;
  function OnMouseDown(Button: TMouseButton; Pos: TVec2f; Flags: Cardinal): Boolean; override;
  procedure Render; override;
  property Items: TListOfString read FItems write FItems;
  property Font: String read GetFontName write SetFont;
  property Color: TVec4f read FColor write FColor;
  property Current: Integer read FCurrent write FCurrent;
end;

TGuiMenuValueBar = class(TWidget)
private
  FMaxValue: Single;
  FValue: Single;
  FOnChange: TProcOfObj;
  function GetValue: Single;
  procedure SetValue(NewValue: Single);
  procedure SetOnChange(NewOnChange: TProcOfObj);
public
  constructor Create(Gui: TGui; Style: String);
  procedure Render; override;
  function OnMouseDown(Button: TMouseButton; Pos: TVec2f; Flags: Cardinal): Boolean; override;
  property MaxValue: Single read FMaxValue write FMaxValue;
  property Value: Single read GetValue write SetValue;
  property OnChange: TProcOfObj read FOnChange write SetOnChange;
end;

TGuiMenuWindow = class(TWidget)
private
  FClose: TGuiMenuButton;
public
  constructor Create(Gui: TGui; Name, Style: String);
  destructor Destroy; override;
  procedure Render; override;
  procedure Update; override;
  procedure OnClose;
end;

implementation

constructor TGuiMenu.Create(Gui: TGui; Name, Style: String);
begin
  inherited Create(Gui, Name, Style);
end;

procedure TGuiMenu.Render;
begin
  inherited Render;
  Style.Color4f(Style.BackColor);
  Style.DrawRect(GlobalRect);
  Style.Color4f(Style.PrimaryColor);
  Style.SetLineWidth(2);
  Style.DrawRect(GlobalRect, False);
end;

procedure TGuiMenu.Update;
begin
  if Parent = nil then
    Exit;
  if IsMouseIn then begin
    Local := Vec2f(Max(Parent.Size.X - Size.X, Local.X - 4), 
                   (Parent.Size.Y - Size.Y)/2) 
  end else begin
    Local := Vec2f(Min(Parent.Size.X - 40, Local.X + 4), 
                   (Parent.Size.Y - Size.Y)/2)
  end;
end;

constructor TGuiMenuButton.Create;
begin
  inherited Create(Gui, ClassName, Style);
  FLabel := TLabel.Create(Self);
  FLabel.Color := Self.Style.PrimaryColor;
  FLabel.Text := Caption;
  FOnClick := nil;
end;

procedure TGuiMenuButton.Render; 
  var
    R: TRect;
begin
  inherited;
  Style.Color4f(FLabel.Color + Ord(IsMouseIn) * Vec4f(0.2, 0.2, 0.2, 0));
  FLabel.DrawLabel;
  //Style.DrawRect(GlobalRect, False);
  {if IsMouseIn then begin
    Style.SetLineWidth(2);
    R := FLabel.GetTextRect;
    Style.DrawLine(Vec2f(R.A.X, R.B.Y), R.B);
  end;}
end;

function TGuiMenuButton.OnMouseDown;
begin
  Result := False;
  if not IsGlobalPointIn(Pos) then
    Exit;
  //if Button = mbLeft then begin
    Result := True;
    if @FOnClick <> nil then
      FOnClick;
  //end;
end;

function TGuiMenuRadioBox.GetFontName: String;
begin
  Result := '<undefined>';
  if FFont <> nil then
    Result := FFont.Name;
end;

procedure TGuiMenuRadioBox.SetFont(Name: String);
begin
  FFont := Style.FindFont(Name);
end;

constructor TGuiMenuRadioBox.Create(Gui: TGui; Style: String);
begin
  inherited Create(Gui, 'menu_radio_box', Style);
  FItems := TListOfString.Create;
  Font := 'default';
  FColor := Self.Style.PrimaryColor;
  FCurrent := 0;
end;

destructor TGuiMenuRadioBox.Destroy;
begin
  FItems.Free;
end;

function TGuiMenuRadioBox.OnMouseDown;
begin
  Result := False;
  if not IsGlobalPointIn(Pos) then
    Exit;
  FCurrent := Round((Pos.Y - Global.Y)*FItems.Count/Size.Y - 0.01);
  FCurrent := Min(FItems.Last, Max(0, FCurrent));
  Result := True;
end;

procedure TGuiMenuRadioBox.Render;
  var
    I: Integer;
    Y: Single;
begin
  //if FFont = nil then
  //  Exit;
  Style.Color4f(FColor);
  for I := 0 to FItems.Last do begin
    Y := Global.Y + Size.Y*I/FItems.Count;
    Style.DrawText(Vec2f(Global.X + 8, Y), Vec2f(0, 0), FFont, FItems[I]);
    Style.DrawCircle(Vec2f(Global.X + 2, Y + FFont.GetHeight(FItems[I])/2),
                     5, False);
    Style.DrawCircle(Vec2f(Global.X + 2, Y + FFont.GetHeight(FItems[I])/2),
                     5, I = FCurrent);
  end;
end;

function TGuiMenuValueBar.GetValue: Single;
begin
  Result := FValue;
end;

procedure TGuiMenuValueBar.SetValue(NewValue: Single);
begin
  FValue := NewValue;
  if @FOnChange <> nil then
    FOnChange;
end;

procedure TGuiMenuValueBar.SetOnChange(NewOnChange: TProcOfObj);
begin
  FOnChange := NewOnChange;
  if @FOnChange <> nil then
    FOnChange;
end;

constructor TGuiMenuValueBar.Create(Gui: TGui; Style: String);
begin
  inherited Create(Gui, 'manu_value_bar', Style);
  FValue := 1;
  FMaxValue := 1;
  FOnChange := nil;
end;

procedure TGuiMenuValueBar.Render;
begin
  Style.Color4f(Style.PrimaryColor);
  Style.SetLineWidth(2);
  Style.DrawRect(GlobalRect, False);
  Style.DrawRect(Rect(Global, Global + Vec2f(FValue*Size.X, Size.Y)));
end;

function TGuiMenuValueBar.OnMouseDown;
begin
  Result := False;
  if not IsGlobalPointIn(Pos) then
    Exit;
  FValue := ((Pos.X - Global.X)*FMaxValue)/Size.X;
  if @FOnChange <> nil then
    FOnChange;
  Result := True;
end;

constructor TGuiMenuWindow.Create(Gui: TGui; Name, Style: String);
begin
  inherited Create(Gui, Name, Style);
  FClose := TGuiMenuButton.Create(Gui, Style, 'close_button');
  FClose.OnClick := OnClose;
  FClose._Label.Font := 'large';
  FClose._Label.Text := 'x';
  FClose.Parent := Self;
end;

destructor TGuiMenuWindow.Destroy;
begin
  inherited;
end;

procedure TGuiMenuWindow.Render;
begin
  inherited Render;
  Style.Color4f(Style.BackColor);
  Style.DrawRect(GlobalRect);
  Style.Color4f(Style.PrimaryColor);
  Style.SetLineWidth(2);
  Style.DrawRect(GlobalRect, False);
end;

procedure TGuiMenuWindow.Update;
begin
  inherited Update;
  FClose.Local := Vec2f(Size.X - 60, 0);
  FClose.Size := Vec2f(60, 50);
end;

procedure TGuiMenuWindow.OnClose;
begin
  Show := False;
end;

end.
